@@ -18,7 +18,8 @@
  * Rights Reserved.
  *
  * Contributor(s):
- *  Robert Ginda, rginda@ndcico.com, original author
+ *  Robert Ginda, rginda@netscape.com, original author
+ *  Samuel Sieb, samuel@sieb.net
  *  Chiaki Koufugata chiaki@mozilla.gr.jp UI i18n 
  */
 
@@ -57,7 +58,7 @@ function onTopicKeyPress (e)
     if (e.keyCode == 13)
     {        
         var line = stringTrim(e.target.value);
-        client.currentObject.setTopic (line);
+        client.currentObject.setTopic (fromUnicode(line));
         onTopicEditEnd();
     }
 }
@@ -79,8 +80,6 @@ function onLoad()
     initHost(client);
     readIRCPrefs();
     setClientOutput();
-    frames[0].document.location.href =
-        "chrome://chatzilla/content/outputwindow.html?" + client.DEFAULT_STYLE;
     initStatic();
 
     client.userScripts = new Array();
@@ -94,6 +93,7 @@ function onLoad()
         }
     }
     
+    processStartupURLs();
     mainStep();
 }
 
@@ -373,7 +373,7 @@ function onOutputContextMenuCreate(e)
         {
             format = format.replace (/\$nick/gi, targetProperNick);
             format = format.replace (/\$viewname/gi,
-                                     client.currentObject.name);
+                                     client.currentObject.unicodeName);
             menuitem.setAttribute ("label", format);
         }
         
@@ -467,6 +467,15 @@ function onToggleVisibility(thing)
     
     if (d == "true")
     {
+        if (thing == "info")
+        {
+            if (client.currentObject.TYPE == "IRCChannel")
+                client.rdf.setTreeRoot ("user-list", 
+                                        client.currentObject.getGraphResource());
+            else
+                client.rdf.setTreeRoot ("user-list", client.rdf.resNullChan);
+        }
+        
         newState = "false";
         menu.setAttribute ("checked", "true");
         client.uiState[thing] = true;
@@ -502,15 +511,17 @@ function onDoStyleChange (newStyle)
     
 }
 
-function onHideCurrentView()
+function onHideView(view)
 {
-    var tb = getTabForObject(client.currentObject);
+    var tb = getTabForObject(view);
     
     if (tb)
     {
         var i = deleteTab (tb);
         if (i != -1)
         {
+            client.deck.removeChild(view.frame);
+            delete view.frame;
             if (i >= client.viewsArray.length)
                 i = client.viewsArray.length - 1;
             
@@ -536,6 +547,8 @@ function onDeleteView(view)
         {
             delete view.messageCount;
             delete view.messages;
+            client.deck.removeChild(view.frame);
+            delete view.frame;
 
             if (i >= client.viewsArray.length)
                 i = client.viewsArray.length - 1;            
@@ -549,15 +562,12 @@ function onDeleteView(view)
 function onClearCurrentView()
 {
 
-    if (client.output.firstChild)
-        client.output.removeChild (client.output.firstChild);
-
     client.currentObject.messages = null;
     client.currentObject.messageCount = 0;
 
     client.currentObject.display (getMsg("onClearCurrentViewMsg"), "INFO");
 
-    client.output.appendChild (client.currentObject.messages);
+    client.currentObject.frame.reload();
     
 }
 
@@ -620,11 +630,11 @@ function onSortCol(sortColName)
         return false;
     try
     {
-        xulSortService.Sort(node, sortResource, sortDirection);
+        xulSortService.sort(node, sortResource, sortDirection);
     }
     catch(ex)
     {
-            //dd("Exception calling xulSortService.Sort()");
+            //dd("Exception calling xulSortService.sort()");
     }
     
     return false;
@@ -698,7 +708,7 @@ function onToggleStartupURL()
 
 function onViewMenuShowing ()
 {
-    var loc = frames[0].document.location.href;
+    var loc = client.currentFrame.document.location.href;
     loc = loc.substr (loc.indexOf("?") + 1);
 
     var val = (loc == "chrome://chatzilla/skin/output-default.css");
@@ -812,7 +822,7 @@ function onTabCompleteRequest (e)
         return;
     }
 
-    var wordStart = line.substr(0, selStart).search(/\s\w*$/);
+    var wordStart = line.substr(0, selStart).search(/\s\S*$/);
     if (wordStart == -1)
         wordStart = 0;
     else
@@ -910,7 +920,7 @@ function onWindowKeyPress (e)
             break;
 
         case 33: /* pgup */
-            w = window.frames[0];
+            w = client.currentFrame;
             newOfs = w.pageYOffset - (w.innerHeight / 2);
             if (newOfs > 0)
                 w.scrollTo (w.pageXOffset, newOfs);
@@ -920,7 +930,7 @@ function onWindowKeyPress (e)
             break;
             
         case 34: /* pgdn */
-            w = window.frames[0];
+            w = client.currentFrame;
             newOfs = w.pageYOffset + (w.innerHeight / 2);
             if (newOfs < (w.innerHeight + w.pageYOffset))
                 w.scrollTo (w.pageXOffset, newOfs);
@@ -1045,7 +1055,7 @@ function cli_icommand (e)
     if (ary.length == 0)
     {        
         var o = getObjectDetails(client.currentObject);
-        if (o.server)
+        if ("server" in o)
         {
             client.currentObject.display (getMsg("cli_icommandMsg",
                                                  e.command), "WARNING");
@@ -1084,25 +1094,29 @@ function cli_icommand (e)
 client.onInputCSS =
 function cli_icss (e)
 {
-    if (!e.inputData)
-        return false;
-
-    e.inputData = stringTrim(e.inputData);
+    if (e.inputData)
+    {
+        e.inputData = stringTrim(e.inputData);
     
-    if (e.inputData.search(/^light$/i) != -1)
-        e.inputData = "chrome://chatzilla/skin/output-light.css";
-    else if (e.inputData.search(/^dark$/i) != -1)
-        e.inputData = "chrome://chatzilla/skin/output-dark.css";
-    else if (e.inputData.search(/^default$/i) != -1)
-        e.inputData = "chrome://chatzilla/skin/output-default.css";
-    else if (e.inputData.search(/^none$/i) != -1)
-        e.inputData = "chrome://chatzilla/content/output-base.css";
+        if (e.inputData.search(/^light$/i) != -1)
+            e.inputData = "chrome://chatzilla/skin/output-light.css";
+        else if (e.inputData.search(/^dark$/i) != -1)
+            e.inputData = "chrome://chatzilla/skin/output-dark.css";
+        else if (e.inputData.search(/^default$/i) != -1)
+            e.inputData = "chrome://chatzilla/skin/output-default.css";
+        else if (e.inputData.search(/^none$/i) != -1)
+            e.inputData = "chrome://chatzilla/content/output-base.css";
     
-    client.currentObject.display (getMsg("cli_icss", e.inputData), "INFO");
+        for (var i = 0; i < client.deck.childNodes.length; i++)
+        {
+            client.deck.childNodes[i].loadURI (
+                "chrome://chatzilla/content/outputwindow.html?" + e.inputData);
+        }
+        client.DEFAULT_STYLE = e.inputData;
+    }
     
-    frames[0].document.location.href =
-        "chrome://chatzilla/content/outputwindow.html?" + e.inputData;
-    client.DEFAULT_STYLE = e.inputData;
+    client.currentObject.display (getMsg("cli_icss", client.DEFAULT_STYLE),
+                                  "INFO");
     return true;
 }
 
@@ -1119,7 +1133,36 @@ function cli_iscommand (e)
     else
     {
         client.currentObject.display (getMsg("onInputSimpleCommandMsg",
-                                             e.command),"WARNING");
+                                             e.command), "WARNING");
+        return false;
+    }
+}
+
+client.onInputSquery =
+function cli_isquery (e)
+{
+    var o = getObjectDetails(client.currentObject);
+    
+    if ("server" in o)
+    {
+        var ary = e.inputData.match (/(\S+)? ?(.*)/);
+        if (ary == null)
+            return false;
+
+        if (ary.length == 1)
+        {
+            o.server.sendData ("SQUERY " + ary[1] + "\n");
+        }
+        else
+        {
+            o.server.sendData ("SQUERY " + ary[1] + " :" + ary[2] + "\n");
+        }
+        return true;
+    }
+    else
+    {
+        client.currentObject.display (getMsg("onInputSimpleCommandMsg",
+                                             e.command), "WARNING");
         return false;
     }
 }
@@ -1144,10 +1187,11 @@ function cli_istatus (e)
                                              s.connection.port, serverType]),
                                      "STATUS");        
 
-        var connectTime = formatDateOffset ((new Date() -
-                                             s.connection.connectDate) / 1000);
-        var pingTime = (s.lastPing) ?
-            formatDateOffset ((new Date() - s.lastPing) / 1000) : getMsg("na");
+        var connectTime = formatDateOffset (Math.floor((new Date() -
+                                            s.connection.connectDate) / 1000));
+        var pingTime = ("lastPing" in s) ?
+            formatDateOffset (Math.floor((new Date() - s.lastPing) / 1000)) :
+            getMsg("na");
         var lag = (s.lag >= 0) ? s.lag : getMsg("na");
 
         client.currentObject.display(getMsg("cli_istatusServerDetail", 
@@ -1178,26 +1222,27 @@ function cli_istatus (e)
                 mode = getMsg("cli_istatusNoMode");
             
             client.currentObject.display (getMsg("cli_istatusChannelOn",
-                                                 [net, mtype, c.name, mode,
+                                                 [net, mtype, c.unicodeName,
+                                                  mode,
                                                   "irc://" + escape(net) + "/" +
-                                                  escape(c.name) + "/"]),
+                                                  escape(c.encodedName) + "/"]),
                                           "STATUS");
             client.currentObject.display (getMsg("cli_istatusChannelDetail",
-                                                 [net, c.name,
+                                                 [net, c.unicodeName,
                                                   c.getUsersLength(),
                                                   c.opCount, c.voiceCount]),
                                           "STATUS");
             if (c.topic)
                 client.currentObject.display (getMsg("cli_istatusChannelTopic",
-                                                     [net, c.name, c.topic]),
+                                              [net, c.unicodeName, c.topic]),
                                               "STATUS");
             else
                 client.currentObject.display (getMsg("cli_istatusChannelNoTopic",
-                                                     [net, c.name]), "STATUS");
+                                              [net, c.unicodeName]), "STATUS");
         }
         else
             client.currentObject.display (getMsg("cli_istatusChannelOff",
-                                                 [net, c.name]), "STATUS");
+                                          [net, c.unicodeName]), "STATUS");
     }
 
     client.currentObject.display (client.userAgent, "STATUS");
@@ -1413,7 +1458,7 @@ function cli_iserver (e)
         return false;
 
     var ary = 
-        e.inputData.match(/^([^\s\:]+)[\s\:]?(?:(\d+)(?:\s+(\S+)|$)|(\S+)|$)/);
+        e.inputData.match(/^([^\s\:]+)[\s\:]?(?:(\d+)(?:\s+(\S+)|\s*$)|(\S+)|$)/);
     var pass;
     if (3 in ary && ary[3])
         pass = ary[3];
@@ -1513,7 +1558,7 @@ client.onInputHide=
 function cli_ihide (e)
 {
     
-    onHideCurrentView();
+    onHideView(client.currentObject);
     return true;
 
 }
@@ -1547,7 +1592,10 @@ function cli_inames (e)
             return false;
         }
 
-        chan = e.inputData;
+        var encodeName = fromUnicode(e.inputData + " ");
+        encodeName = encodeName.substr(0, encodeName.length -1);
+        chan = encodeName;
+
     }
     else
     {
@@ -1662,6 +1710,51 @@ function cli_ime (e)
     return true;
 }
 
+client.onInputList =
+function cli_ilist (e)
+{
+    var o = getObjectDetails(client.currentObject);
+    if ("network" in o)
+    {
+        o.network.list = new Array();
+        o.network.list.regexp = null;
+    }
+    return client.onInputSimpleCommand(e);
+}
+
+client.onInputRlist =
+function cli_irlist (e)
+{
+    var o = getObjectDetails(client.currentObject);
+
+    if (!("server" in o))
+    {
+        client.currentObject.display (getMsg("onInputSimpleCommandMsg",
+                                             "list"), "WARNING");
+        return false;
+    }
+    o.network.list = new Array();
+    var ary = e.inputData.match (/^\s*("([^"]*)"|([^\s]*))/);
+    try
+    {
+        if (ary[2])
+            o.network.list.regexp = new RegExp(ary[2], "i");
+        else if (ary[3])
+            o.network.list.regexp = new RegExp(ary[3], "i");
+        else
+            return false;
+    }
+    catch(error)
+    {
+        client.currentObject.display (getMsg("cli_irlistMsg", e.inputData,
+                                             error),
+                                      "ERROR");
+        return false;
+    }
+    o.server.sendData ("list\n");
+    return true;
+}
+
 client.onInputQuery =
 function cli_iquery (e)
 {
@@ -1839,39 +1932,46 @@ function cli_ijoin (e)
         return false;
     }
     
-    var ary = e.inputData.match(/(\S+) ?(\S+)?/);
+    var ary = e.inputData.match(/(((\S+), *)*(\S+)) *(\S+)?/);
     var name;
     var key = "";
-    
+    var namelist;
+
     if (ary)
     {
-        name = ary[1];
-        if (2 in ary)
-            key = ary[2];
+        namelist = ary[1].split(/, */);
+        if (5 in ary)
+            key = ary[5];
     }
     else
     {
         if (client.currentObject.TYPE == "IRCChannel")
-            name = client.currentObject.name;
+            namelist = [client.currentObject.name];
         else
             return false;
 
         if (client.currentObject.mode.key)
             key = client.currentObject.mode.key
     }
-    
-    if ((name[0] != "#") && (name[0] != "&") && (name[0] != "+") &&
-        (name[0] != "!"))
-        name = "#" + name;
 
-    e.channel = e.server.addChannel (name);
-    e.channel.join(key);
-    if (!("messages" in e.channel))
-        e.channel.display (getMsg("cli_ijoinMsg3",e.channel.name), "INFO");
-    setCurrentObject(e.channel);
-    
+    for (i in namelist)
+    {
+        name = namelist[i];
+        if ((name[0] != "#") && (name[0] != "&") && (name[0] != "+") &&
+            (name[0] != "!"))
+            name = "#" + name;
+
+        e.channel = e.server.addChannel (name);
+        e.channel.join(key);
+        if (!("messages" in e.channel))
+        {
+            e.channel.display (getMsg("cli_ijoinMsg3", e.channel.unicodeName),
+                               "INFO");
+        }
+        setCurrentObject(e.channel);
+    }
+
     return true;
-    
 }
 
 client.onInputLeave =
@@ -2253,8 +2353,10 @@ function cli_iinvite (e)
         }
         else
         {
-            var chan = e.server.channels[ary[1].toLowerCase()];
-
+            var encodeName = fromUnicode(ary[1] + " ");
+            encodeName = encodeName.substr(0, encodeName.length -1);
+            var chan = e.server.channels[encodeName.toLowerCase()];
+             
             if (chan == undefined) 
             {
                 client.currentObject.display (getMsg("cli_iinviteMsg3", ary[1]),
@@ -2307,12 +2409,12 @@ function cli_ikick (e)
 
     if (ary.length > 2)
     {               
-        cuser.kick(ary[2]);
+        cuser.kick(fromUnicode(ary[2]));
     }
     else     
 
     cuser.kick();    
-            
+     
     return true;
 }
 
@@ -2631,22 +2733,90 @@ function my_303 (e)
 CIRCNetwork.prototype.on321 = /* LIST reply header */
 function my_321 (e)
 {
+
+    function checkEndList (network)
+    {
+        if (network.list.count == network.list.lastLength)
+        {
+            network.on323();
+        }
+        else
+        {
+            network.list.lastLength = network.list.count;
+            network.list.endTimeout =
+                setTimeout(checkEndList, 1500, network);
+        }
+    }
+
+    function outputList (network)
+    {
+        const CHUNK_SIZE = 5;
+        var list = network.list;
+        if (list.length > list.displayed)
+        {
+            var start = list.displayed;
+            var end = list.length;
+            if (end - start > CHUNK_SIZE)
+                end = start + CHUNK_SIZE;
+            for (var i = start; i < end; ++i)
+                network.displayHere (getMsg("my_322", list[i]), "322");
+            list.displayed = end;
+        }
+        if (list.done && (list.displayed == list.length))
+        {
+            if (list.event323)
+            {
+                network.displayHere (list.event323.params.join(" ") + ": " +
+                                     list.event323.meat, "323");
+            }
+            network.displayHere (getMsg("my_323", [list.displayed, list.count]),
+                                 "INFO");
+            delete network.list;
+        }
+        else
+        {
+            setTimeout(outputList, 250, network);
+        }
+    }
+
+    if (!("list" in this))
+    {
+        this.list = new Array();
+        this.list.regexp = null;
+    }
     this.displayHere (e.params[2] + " " + e.meat, "321");
     if (client.currentObject != this)
-    client.currentObject.display (getMsg("my_321", this.name), "INFO");
+        client.currentObject.display (getMsg("my_321", this.name), "INFO");
+    this.list.lastLength = 0;
+    this.list.done = false;
+    this.list.count = 0;
+    this.list.displayed = 0;
+    setTimeout(outputList, 250, this);
+    this.list.endTimeout = setTimeout(checkEndList, 1500, this);
 }
 
 CIRCNetwork.prototype.on323 = /* end of LIST reply */
 function my_323 (e)
 {
-    this.displayHere (e.params.join(" ") + ": " + e.meat, "323");
+    if (this.list.endTimeout)
+    {
+        clearTimeout(this.list.endTimeout);
+        delete this.list.endTimeout;
+    }
+    this.list.done = true;
+    this.list.event323 = e;
 }
 
 CIRCNetwork.prototype.on322 = /* LIST reply */
 function my_listrply (e)
 {
-    this.displayHere (getMsg("my_322", [e.params[2], e.params[3], e.meat]),
-                      "322");
+    ++this.list.count;
+    e.params[2] = toUnicode(e.params[2]);
+    if (!(this.list.regexp) || e.params[2].match(this.list.regexp)
+                            || e.meat.match(this.list.regexp))
+    {
+        this.list.push([e.params[2], e.params[3], e.meat]);
+    }
 }
 
 /* end of WHO */
@@ -2665,7 +2835,8 @@ function my_315 (e)
 CIRCNetwork.prototype.on352 =
 function my_352 (e)
 {
-    //0-352 1-rginda_ 2-#chatzilla 3-chatzilla 4-h-64-236-139-254.aoltw.net 5-irc.mozilla.org 6-rginda 7-H
+    //0-352 1-rginda_ 2-#chatzilla 3-chatzilla 4-h-64-236-139-254.aoltw.net
+    //5-irc.mozilla.org 6-rginda 7-H
     var desc;
     var hops = "?";
     var ary = e.meat.match(/(\d+)\s(.*)/);
@@ -2686,8 +2857,8 @@ function my_352 (e)
         status = getMsg("my_352.h");
         
     e.user.display (getMsg("my_352", [e.params[6], e.params[3], e.params[4],
-                                      desc, status, e.params[2], e.params[5],
-                                      hops]), e.code, e.user);
+                                      desc, status, toUnicode(e.params[2]), 
+                                      e.params[5], hops]), e.code, e.user);
     updateTitle (e.user);
     if ("whoMatches" in this)
         ++this.whoMatches;
@@ -2749,7 +2920,7 @@ function my_whoisreply (e)
 CIRCNetwork.prototype.on341 = /* invite reply */
 function my_341 (e)
 {
-    this.display (getMsg("my_341", [e.params[2], e.params[3]]), "341");
+    this.display (getMsg("my_341", [e.params[2], toUnicode(e.params[3])]), "341");
 }
 
 CIRCNetwork.prototype.onInvite = /* invite message */
@@ -3000,10 +3171,10 @@ function my_topic (e)
     if (e.code == "332")
     {
         if (this.topic)
-            this.display (getMsg("my_topicMsg2", [this.name, this.topic]),
+            this.display (getMsg("my_topicMsg2", [this.unicodeName, this.topic]),
                           "TOPIC");
         else
-            this.display (getMsg("my_topicMsg3", this.name), "TOPIC");
+            this.display (getMsg("my_topicMsg3", this.unicodeName), "TOPIC");
     }
     
     updateChannel (this);
@@ -3015,7 +3186,7 @@ CIRCChannel.prototype.on333 = /* Topic setter information */
 function my_topicinfo (e)
 {
     
-    this.display (getMsg("my_topicinfoMsg", [this.name, this.topicBy, 
+    this.display (getMsg("my_topicinfoMsg", [this.unicodeName, this.topicBy, 
                                              this.topicDate]), "TOPIC");
     
 }
@@ -3058,13 +3229,13 @@ function my_cjoin (e)
 
     if (userIsMe (e.user))
     {
-        this.display (getMsg("my_cjoinMsg", e.channel.name), "JOIN",
+        this.display (getMsg("my_cjoinMsg", e.channel.unicodeName), "JOIN",
                       e.server.me, this);
         setCurrentObject(this);
     }
     else
         this.display(getMsg("my_cjoinmsg2", [e.user.properNick, e.user.name,
-                                             e.user.host, e.channel.name]),
+                                             e.user.host, e.channel.unicodeName]),
                      "JOIN", e.user, this);
 
     this._addUserToGraph (e.user);
@@ -3081,8 +3252,8 @@ function my_cpart (e)
 
     if (userIsMe (e.user))
     {
-        this.display (getMsg("my_cpartMsg", e.channel.name), "PART", e.user,
-                      this);
+        this.display (getMsg("my_cpartMsg", e.channel.unicodeName), "PART",
+                      e.user, this);
         if (client.currentObject == this)    
             /* hide the tree while we remove (possibly tons) of nodes */
             client.rdf.setTreeRoot("user-list", client.rdf.resNullChan);
@@ -3099,7 +3270,7 @@ function my_cpart (e)
     }
     else
         this.display (getMsg("my_cpartMsg2",
-                             [e.user.properNick, e.channel.name]), 
+                             [e.user.properNick, e.channel.unicodeName]), 
                       "PART", e.user, this);
 
     updateChannel (e.channel);
@@ -3112,7 +3283,7 @@ function my_ckick (e)
 
     if (userIsMe (e.lamer))
         this.display (getMsg("my_ckickMsg",
-                             [e.channel.name, e.user.properNick, e.reason]),
+                             [e.channel.unicodeName, e.user.properNick, e.reason]),
                       "KICK", e.user, this);
     else
     {
@@ -3129,8 +3300,8 @@ function my_ckick (e)
         }
         
         this.display (getMsg("my_ckickMsg2",
-                             [e.lamer.properNick, e.channel.name, enforcerProper,
-                             e.reason]), "KICK", e.user, this);
+                             [e.lamer.properNick, e.channel.unicodeName, 
+                              enforcerProper, e.reason]), "KICK", e.user, this);
     }
     
     this._removeUserFromGraph(e.lamer);
@@ -3144,9 +3315,9 @@ function my_cmode (e)
 {
 
     if ("user" in e)
-        this.display (getMsg("my_cmodeMsg", [e.params.slice(1).join(" "),
-                                             e.user.properNick]), "MODE",
-                      e.user, this);
+        this.display (getMsg("my_cmodeMsg",
+                             [toUnicode(e.params.slice(1).join(" ")),
+                              e.user.properNick]), "MODE", e.user, this);
 
     for (var u in e.usersAffected)
         e.usersAffected[u].updateGraphResource();
@@ -3209,8 +3380,8 @@ function my_cprivmsg (e)
     else
     {        
         playSounds(client.MSG_BEEP);
-        if (client.NEW_TAB_THRESHOLD == 0 ||
-            client.viewsArray.length < client.NEW_TAB_THRESHOLD)
+        if (client.NEW_TAB_LIMIT == 0 ||
+            client.viewsArray.length < client.NEW_TAB_LIMIT)
         {
             var tab = openQueryTab (e.server, e.user.nick);
             if (client.FOCUS_NEW_TAB)
@@ -3256,3 +3427,4 @@ function my_unkctcp (e)
                                 "BAD-CTCP", this, e.server.me);
 
 }
+
