@@ -76,7 +76,7 @@ function GetServer(uri)
 
 function LoadMessageByUri(uri)
 {
-    dump("XXX LoadMessageByUri " + uri + " vs " + gCurrentDisplayedMessage + "\n");
+  //dump("XXX LoadMessageByUri " + uri + " vs " + gCurrentDisplayedMessage + "\n");
   if(uri != gCurrentDisplayedMessage)
   {
         dump("fix this, get the nsIMsgDBHdr and the nsIMsgFolder from the uri...\n");
@@ -110,14 +110,8 @@ function setTitleFromFolder(msgfolder, subject)
     else
       title = "";
 
-    if (msgfolder.isServer)
-    {
-      if (server.type == "none")
-        title += server.prettyName;
-      else
-       // <hostname>
-        title += server.hostName;
-    }
+    if (msgfolder.isServer) 
+      title += server.prettyName;
     else {
         var middle;
         var end;
@@ -152,6 +146,10 @@ function UpdateMailToolbar(caller)
 {
   //dump("XXX update mail-toolbar " + caller + "\n");
   document.commandDispatcher.updateCommands('mail-toolbar');
+
+  // hook for extra toolbar items
+  var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
+  observerService.notifyObservers(window, "mail:updateToolbarItems", null);
 }
 
 function ChangeFolderByURI(uri, viewType, viewFlags, sortType, sortOrder)
@@ -159,6 +157,11 @@ function ChangeFolderByURI(uri, viewType, viewFlags, sortType, sortOrder)
   //dump("In ChangeFolderByURI uri = " + uri + " sortType = " + sortType + "\n");
   if (uri == gCurrentLoadingFolderURI)
     return;
+
+  // hook for extra toolbar items
+  var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
+  observerService.notifyObservers(window, "mail:setupToolbarItems", uri);
+
   var resource = RDF.GetResource(uri);
   var msgfolder =
       resource.QueryInterface(Components.interfaces.nsIMsgFolder);
@@ -209,7 +212,22 @@ function ChangeFolderByURI(uri, viewType, viewFlags, sortType, sortOrder)
   gCurrentLoadingFolderViewType = viewType;
   gCurrentLoadingFolderSortType = sortType;
   gCurrentLoadingFolderSortOrder = sortOrder;
-  if(msgfolder.manyHeadersToDownload)
+
+  var showMessagesAfterLoading;
+  try {
+    var server = msgfolder.server;
+    if (server.redirectorType) {
+      var prefString = server.type + "." + server.redirectorType + ".showMessagesAfterLoading";
+      showMessagesAfterLoading = gPrefs.getBoolPref(prefString);
+    }
+    else
+      showMessagesAfterLoading = false;
+  }
+  catch (ex) {
+    showMessagesAfterLoading = false;
+  }
+
+  if (msgfolder.manyHeadersToDownload || showMessagesAfterLoading)
   {
     gRerootOnFolderLoad = true;
     try
@@ -305,15 +323,15 @@ function RerootFolder(uri, newFolder, viewType, viewFlags, sortType, sortOrder)
         oldFolder.setMsgDatabase(null);
   }
   // that should have initialized gDBView, now re-root the thread pane
-  var treeView = gDBView.QueryInterface(Components.interfaces.nsITreeView);
-  if (treeView)
-  {
-    var tree = GetThreadTree();
-    tree.boxObject.QueryInterface(Components.interfaces.nsITreeBoxObject).view = treeView;
-  }
+  RerootThreadPane();
 
   SetUpToolbarButtons(uri);
+
   UpdateStatusMessageCounts(newFolder);
+  
+  // hook for extra toolbar items
+  var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
+  observerService.notifyObservers(window, "mail:updateToolbarItems", null);
 }
 
 function SwitchView(command)
@@ -460,6 +478,12 @@ function ConvertColumnIDToSortType(columnID)
     case "labelCol":
       sortKey = nsMsgViewSortType.byLabel;
       break;
+    case "junkStatusCol":
+      sortKey = nsMsgViewSortType.byJunkStatus;
+      break;
+    case "idCol":
+      sortKey = nsMsgViewSortType.byId;
+      break;
     default:
       dump("unsupported sort column: " + columnID + "\n");
       sortKey = 0;
@@ -512,8 +536,10 @@ function ConvertSortTypeToColumnID(sortKey)
       columnID = "threadCol";
       break;
     case nsMsgViewSortType.byId:
-      // there is no orderReceivedCol, so return null
-      columnID = null;
+      columnID = "idCol";
+      break;
+    case nsMsgViewSortType.byJunkStatus:
+      columnID = "junkStatusCol";
       break;
     default:
       dump("unsupported sort key: " + sortKey + "\n");
@@ -545,6 +571,9 @@ function CreateBareDBView(originalView, msgFolder, viewType, viewFlags, sortType
   viewType = viewType - 0;
 
   switch (viewType) {
+      case nsMsgViewType.eShowQuickSearchResults:
+          dbviewContractId += "quicksearch";
+          break;
       case nsMsgViewType.eShowThreadsWithUnread:
           dbviewContractId += "threadswithunread";
           break;
@@ -629,22 +658,38 @@ function GetSelectedFolderResource()
     return GetFolderResource(folderTree, startIndex.value);
 }
 
+function NotifyChangedMessagePaneVisibility(now_hidden)
+{
+  var event = document.createEvent('Events');
+  if (now_hidden) {
+    event.initEvent('messagepane-hide', false, true);
+  }
+  else {
+    event.initEvent('messagepane-unhide', false, true);
+  }
+  document.getElementById("messengerWindow").dispatchEvent(event);
+}
+
 function OnMouseUpThreadAndMessagePaneSplitter()
-  {
+{
+  // the collapsed state is the state after we released the mouse 
+  // so we take it as it is
+  var now_hidden = IsThreadAndMessagePaneSplitterCollapsed();
   if (gDBView) {
-    // the collapsed state is the state after we released the mouse 
-    // so we take it as it is
-    gDBView.suppressMsgDisplay = IsThreadAndMessagePaneSplitterCollapsed();
-  }
+    gDBView.suppressMsgDisplay = now_hidden;
   }
+  NotifyChangedMessagePaneVisibility(now_hidden);
+}
 
 function OnClickThreadAndMessagePaneSplitterGrippy()
-  {
+{
+  // the collapsed state is the state when we clicked on the grippy
+  // not when afterwards, so we need to reverse this value
+  var now_hidden = !IsThreadAndMessagePaneSplitterCollapsed();
   if (gDBView) {
-    // the collapsed state is the state when we clicked on the grippy
-    // not when afterwards, so we need to reverse this value
-    gDBView.suppressMsgDisplay = !IsThreadAndMessagePaneSplitterCollapsed();
-   }
+    gDBView.suppressMsgDisplay = now_hidden;
+  }
+  NotifyChangedMessagePaneVisibility(now_hidden);
 }
 
 function FolderPaneSelectionChange()
@@ -705,14 +750,23 @@ function FolderPaneSelectionChange()
                 dump("failed to get view & sort values.  ex = " + ex +"\n");
               }
             }
-            if (gDBView && gDBView.isSearchView)
+            if (gDBView && gDBView.viewType == nsMsgViewType.eShowQuickSearchResults)
             {
-              gDBView.isSearchView = false;  //reset the search input on folder switch
-              var searchInput = document.getElementById("searchInput");
+              if (gPreQuickSearchView) //close cached view before quick search
+              {
+                gPreQuickSearchView.close();
+                gPreQuickSearchView = null;  
+              }
+              var searchInput = document.getElementById("searchInput");  //reset the search input on folder switch
               if (searchInput) 
                 searchInput.value = "";
             }
             ClearMessagePane();
+
+            if (gSearchEmailAddress || gDefaultSearchViewTerms)
+              viewType = nsMsgViewType.eShowQuickSearchResults;
+            else if (viewType == nsMsgViewType.eShowQuickSearchResults)
+              viewType = nsMsgViewType.eShowAllThreads;  //override viewType - we don't want to start w/ quick search
             ChangeFolderByURI(folderResource.Value, viewType, viewFlags, sortType, sortOrder);
         }
     }
